@@ -21,18 +21,14 @@
 
 #include "glossary.h"
 #include "view.h"
+#include "khc_debug.h"
 
-#include <KConfig>
-#include <KIconLoader>
-#include <KConfigGroup>
 #include <KXmlGuiWindow>
 #include <KProcess>
 #include <QStatusBar>
 
 #include <QTreeWidgetItem>
 
-#include <QFrame>
-#include <QListView>
 #include <qapplication.h>
 #include <QTextStream>
 
@@ -40,6 +36,9 @@
 #include <QStandardPaths>
 #include <QDir>
 #include <QFileInfo>
+#include <QIcon>
+
+#include <prefs.h>
 
 using namespace KHC;
 
@@ -50,7 +49,7 @@ class SectionItem : public QTreeWidgetItem
 	: QTreeWidgetItem( parent )
         {
 	  setText(0,text);
-	  setIcon(0,SmallIcon( "help-contents" ));
+	  setIcon(0,QIcon::fromTheme( "help-contents" ));
         }
 };
 
@@ -85,17 +84,16 @@ Glossary::Glossary( QWidget *parent ) : QTreeWidget( parent )
 
     m_byTopicItem = new QTreeWidgetItem( this );
     m_byTopicItem->setText( 0, i18n( "By Topic" ) );
-    m_byTopicItem->setIcon( 0, SmallIcon( "help-contents" ) );
+    m_byTopicItem->setIcon( 0, QIcon::fromTheme( "help-contents" ) );
 
     m_alphabItem = new QTreeWidgetItem( this );
     m_alphabItem->setText( 0, i18n( "Alphabetically" ) );
-    m_alphabItem->setIcon( 0, SmallIcon( "character-set" ) );
+    m_alphabItem->setIcon( 0, QIcon::fromTheme( "character-set" ) );
 
     m_cacheFile = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QLatin1Char('/') + "help/glossary.xml" ;
     QDir().mkpath( QFileInfo( m_cacheFile ).absolutePath() );
     
     m_sourceFile = View::langLookup( QLatin1String( "khelpcenter/glossary/index.docbook" ) );
-    m_config = KSharedConfig::openConfig();
 
 }
 
@@ -125,8 +123,8 @@ const GlossaryEntry &Glossary::entry( const QString &id ) const
 Glossary::CacheStatus Glossary::cacheStatus() const
 {
     if ( !QFile::exists( m_cacheFile ) ||
-	m_config->group("Glossary").readPathEntry( "CachedGlossary", QString() ) != m_sourceFile ||
-	m_config->group("Glossary").readEntry( "CachedGlossaryTimestamp" ).toInt() != glossaryCTime() )
+         Prefs::cachedGlossary() != m_sourceFile ||
+         Prefs::cachedGlossaryTimestamp() != glossaryCTime() )
         return NeedRebuild;
 
     return CacheOk;
@@ -160,7 +158,7 @@ void Glossary::rebuildGlossaryCache()
     meinproc->start();
     if (!meinproc->waitForStarted()) 
     {
-        qWarning() << "could not start process" << meinproc->program();
+        khcWarning() << "could not start process" << meinproc->program();
         if (mainWindow && !m_alreadyWarned)
 	{
             ; // add warning message box with don't display again option 
@@ -178,8 +176,8 @@ void Glossary::meinprocFinished( int exitCode, QProcess::ExitStatus exitStatus )
 
     if (exitStatus != QProcess::NormalExit || exitCode != 0)
     {
-        qWarning() << "running" << meinproc->program() << "failed with exitCode" << exitCode;
-        qWarning() << "stderr output:" << meinproc->readAllStandardError();
+        khcWarning() << "running" << meinproc->program() << "failed with exitCode" << exitCode;
+        khcWarning() << "stderr output:" << meinproc->readAllStandardError();
         if (mainWindow && !m_alreadyWarned) 
 	{
             ; // add warning message box with don't display again option 
@@ -194,9 +192,9 @@ void Glossary::meinprocFinished( int exitCode, QProcess::ExitStatus exitStatus )
     if ( !QFile::exists( m_cacheFile ) )
         return;
 
-    m_config->group("Glossary").writePathEntry( "CachedGlossary", m_sourceFile );
-    m_config->group("Glossary").writeEntry( "CachedGlossaryTimestamp", glossaryCTime() );
-    m_config->sync();
+    Prefs::setCachedGlossary( m_sourceFile );
+    Prefs::setCachedGlossaryTimestamp( glossaryCTime() );
+    Prefs::self()->save();
 
     m_status = CacheOk;
 
@@ -269,7 +267,7 @@ void Glossary::buildGlossaryTree()
                     seeAlso += GlossaryEntryXRef( term, id );
                 }
 
-            m_glossEntries.insert( entryId, new GlossaryEntry( term, definition, seeAlso ) );
+            m_glossEntries.insert( entryId, new GlossaryEntry( entryId, term, definition, seeAlso ) );
         }
     }
     sortItems(0, Qt::AscendingOrder);
@@ -295,40 +293,6 @@ QDomElement Glossary::childElement( const QDomElement &element, const QString &n
     return e;
 }
 
-QString Glossary::entryToHtml( const GlossaryEntry &entry )
-{
-    QFile htmlFile( QStandardPaths::locate(QStandardPaths::GenericDataLocation, "khelpcenter/glossary.html.in" ) );
-    if (!htmlFile.open(QIODevice::ReadOnly))
-      return QString( "<html><head></head><body><h3>%1</h3>%2</body></html>" )
-             .arg( i18n( "Error" ) )
-             .arg( i18n( "Unable to show selected glossary entry: unable to open "
-                          "file 'glossary.html.in'!" ) );
-
-    QString seeAlso;
-    if (!entry.seeAlso().isEmpty()) 
-    {
-        seeAlso = i18n("See also: ");
-        GlossaryEntryXRef::List seeAlsos = entry.seeAlso();
-        GlossaryEntryXRef::List::ConstIterator it = seeAlsos.constBegin();
-        GlossaryEntryXRef::List::ConstIterator end = seeAlsos.constEnd();
-        for (; it != end; ++it)
-	{
-            seeAlso += QLatin1String("<a href=\"glossentry:");
-            seeAlso += (*it).id();
-            seeAlso += QLatin1String("\">") + (*it).term();
-            seeAlso += QLatin1String("</a>, ");
-        }
-        seeAlso = seeAlso.left(seeAlso.length() - 2);
-    }
-    
-    QTextStream htmlStream(&htmlFile);
-    return htmlStream.readAll()
-           .arg( i18n( "KDE Glossary" ) )
-           .arg( entry.term() )
-           .arg( entry.definition() )
-           .arg( seeAlso );
-}
-
 void Glossary::slotSelectGlossEntry( const QString &id )
 {
     if ( !m_idDict.contains( id ) )
