@@ -22,72 +22,53 @@
 
 #include <QDir>
 #include <QFile>
-#include <QPixmap>
-
-#include <QLabel>
-#include <QtXml/QtXml>
 #include <QTextStream>
-#include <QRegExp>
-#include <QLayout>
 #include <QPushButton>
-#include <QFrame>
 #include <QHBoxLayout>
-#include <QBoxLayout>
 #include <QVBoxLayout>
 #include <QTreeWidgetItemIterator>
 #include <qapplication.h>
-
-#include <KConfig>
+#include <QProgressBar>
+#include <QTabWidget>
+#include <QTreeWidget>
 
 #include <KLineEdit>
 #include <KMessageBox>
-#include <KIconLoader>
-#include <KCharsets>
-#include <KDialog>
 #include <KDesktopFile>
 #include <KProtocolInfo>
 #include <KServiceGroup>
 #include <KServiceTypeTrader>
-#include <KCModuleInfo>
+#include <KProcess>
+#include <KShell>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include <QStandardPaths>
 
 #include "navigatoritem.h"
 #include "navigatorappitem.h"
+#include "navigatorappgroupitem.h"
 #include "searchwidget.h"
 #include "searchengine.h"
+#include "searchhandler.h"
 #include "docmetainfo.h"
 #include "docentrytraverser.h"
 #include "glossary.h"
 #include "toc.h"
 #include "view.h"
 #include "infotree.h"
-#include "mainwindow.h"
 #include "plugintraverser.h"
 #include "scrollkeepertreebuilder.h"
-#include "kcmhelpcenter.h"
-#include "formatter.h"
 #include "history.h"
-#include "prefs.h"
+#include "khc_debug.h"
+#include "grantleeformatter.h"
+
+#include <prefs.h>
 
 using namespace KHC;
 
-namespace {
-QLoggingCategory category("org.kde.khelpcenter");
-}
-
-Navigator::Navigator( View *view, QWidget *parent, const char *name )
-   : QWidget( parent ), mIndexDialog( 0 ),
-     mView( view ), mSelected( false )
+Navigator::Navigator( View *view, QWidget *parent )
+   : QWidget( parent ),
+     mView( view ), mSelected( false ), mIndexingProc( 0 )
 {
-    setObjectName( name );
-
-    KConfigGroup config(KSharedConfig::openConfig(), "General");
-    mShowMissingDocs = config.readEntry("ShowMissingDocs", false);
-
     mSearchEngine = new SearchEngine( view );
     connect(mSearchEngine, &SearchEngine::searchFinished, this, &Navigator::slotSearchFinished);
 
@@ -95,41 +76,46 @@ Navigator::Navigator( View *view, QWidget *parent, const char *name )
 
     QBoxLayout *topLayout = new QVBoxLayout( this );
 
-    mSearchFrame = new QFrame( this );
-    topLayout->addWidget( mSearchFrame );
+    mSearchLineEditWidget = new QWidget( this );
+    topLayout->addWidget( mSearchLineEditWidget );
 
-    QBoxLayout *searchLayout = new QHBoxLayout( mSearchFrame );
-    searchLayout->setSpacing( KDialog::spacingHint() );
+    QBoxLayout *searchLayout = new QHBoxLayout( mSearchLineEditWidget );
     searchLayout->setMargin( 6 );
 
-    mSearchEdit = new KLineEdit( mSearchFrame );
+    mSearchEdit = new KLineEdit( mSearchLineEditWidget );
     mSearchEdit->setClearButtonShown(true);
     searchLayout->addWidget( mSearchEdit );
     connect(mSearchEdit, &KLineEdit::returnPressed, this, &Navigator::slotSearch);
     connect(mSearchEdit, &KLineEdit::textChanged, this, &Navigator::checkSearchButton);
 
-    mSearchButton = new QPushButton( i18n("&Search"), mSearchFrame );
+    mSearchButton = new QPushButton( i18n("&Search"), mSearchLineEditWidget );
     searchLayout->addWidget( mSearchButton );
     connect(mSearchButton, &QPushButton::clicked, this, &Navigator::slotSearch);
 
     mTabWidget = new QTabWidget( this );
     topLayout->addWidget( mTabWidget );
 
+    mIndexingBar = new QProgressBar( this );
+    mIndexingBar->hide();
+    topLayout->addWidget( mIndexingBar );
+
+    mIndexingTimer.setSingleShot( true );
+    mIndexingTimer.setInterval( 1000 );
+    connect( &mIndexingTimer, &QTimer::timeout, this, &Navigator::slotShowIndexingProgressBar );
+
     setupContentsTab();
     setupGlossaryTab();
     setupSearchTab();
 
     insertPlugins();
-    hideSearch();
-/*
+
     if ( !mSearchEngine->initSearchHandlers() ) {
       hideSearch();
     } else {
       mSearchWidget->updateScopeList();
       mSearchWidget->readConfig( KSharedConfig::openConfig().data() );
+      QTimer::singleShot( 0, this, &Navigator::slotDelayedIndexingStart );
     }
-    */
-    connect(mTabWidget, &QTabWidget::currentChanged, this, &Navigator::slotTabChanged);
 }
 
 Navigator::~Navigator()
@@ -142,16 +128,6 @@ SearchEngine *Navigator::searchEngine() const
   return mSearchEngine;
 }
 
-Formatter *Navigator::formatter() const
-{
-  return mView->formatter();
-}
-
-bool Navigator::showMissingDocs() const
-{
-  return mShowMissingDocs;
-}
-
 void Navigator::setupContentsTab()
 {
     mContentsTree = new QTreeWidget( mTabWidget );
@@ -161,6 +137,8 @@ void Navigator::setupContentsTab()
     mContentsTree->headerItem()->setHidden(true);
 
     connect(mContentsTree, &QTreeWidget::itemActivated, this, &Navigator::slotItemSelected);
+    connect(mContentsTree, &QTreeWidget::itemExpanded, this, &Navigator::slotItemExpanded);
+    connect(mContentsTree, &QTreeWidget::itemCollapsed, this, &Navigator::slotItemCollapsed);
     
     mTabWidget->addTab(mContentsTree, i18n("&Contents"));
 }
@@ -171,7 +149,6 @@ void Navigator::setupSearchTab()
     mSearchWidget = new SearchWidget( mSearchEngine, mTabWidget );
     connect(mSearchWidget, &SearchWidget::searchResult, this, &Navigator::slotShowSearchResult);
     connect(mSearchWidget, &SearchWidget::scopeCountChanged, this, &Navigator::checkSearchButton);
-    connect(mSearchWidget, &SearchWidget::showIndexDialog, this, &Navigator::showIndexDialog);
 
     mTabWidget->addTab( mSearchWidget, i18n("Search Options"));
     
@@ -192,7 +169,7 @@ void Navigator::insertPlugins()
 
 void Navigator::insertParentAppDocs( const QString &name, NavigatorItem *topItem )
 {
-  qCDebug(category) << "Requested plugin documents for ID " << name;
+  khcDebug() << "Requested plugin documents for ID " << name;
 
   KServiceGroup::Ptr grp = KServiceGroup::childGroup( name );
   if ( !grp )
@@ -211,7 +188,7 @@ void Navigator::insertParentAppDocs( const QString &name, NavigatorItem *topItem
 
 void Navigator::insertKCMDocs( const QString &name, NavigatorItem *topItem, const QString &type )
 {
-  qCDebug(category) << "Requested KCM documents for ID" << name;
+  khcDebug() << "Requested KCM documents for ID" << name;
   QString systemsettingskontrolconstraint = "[X-KDE-System-Settings-Parent-Category] != ''";
   QString konquerorcontrolconstraint = "[X-KDE-PluginKeyword] == 'khtml_general'\
                                      or [X-KDE-PluginKeyword] == 'performance'\
@@ -255,8 +232,7 @@ History                 no X-KDE-PluginKeyword in kcmhistory.desktop
   for ( KService::List::const_iterator it = list.constBegin(); it != list.constEnd(); ++it )
   {
     KService::Ptr s(*it);
-    KCModuleInfo m(s);
-    QString desktopFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + m.fileName() );
+    QString desktopFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + s->entryPath() );
     createItemFromDesktopFile( topItem, desktopFile );
     }
     topItem->sortChildren( 0, Qt::AscendingOrder /* ascending */ );
@@ -264,7 +240,7 @@ History                 no X-KDE-PluginKeyword in kcmhistory.desktop
 
 void Navigator::insertIOSlaveDocs( const QString &name, NavigatorItem *topItem )
 {
-  qCDebug(category) << "Requested IOSlave documents for ID" << name;
+  khcDebug() << "Requested IOSlave documents for ID" << name;
 
   QStringList list = KProtocolInfo::protocols();
   list.sort();
@@ -280,7 +256,7 @@ void Navigator::insertIOSlaveDocs( const QString &name, NavigatorItem *topItem )
       QString icon = KProtocolInfo::icon(*it);
       if ( icon.isEmpty() ) icon = "text-plain";
       DocEntry *entry = new DocEntry( *it, url.url(), icon );
-      NavigatorItem *item = new NavigatorItem( entry, topItem, prevItem );
+      NavigatorItem *item = new NavigatorAppItem( entry, topItem, prevItem );
       prevItem = item;
       item->setAutoDeleteDocEntry( true );
     }
@@ -298,7 +274,7 @@ void Navigator::createItemFromDesktopFile( NavigatorItem *topItem,
       QString icon = desktopFile.readIcon();
       if ( icon.isEmpty() ) icon = "text-plain";
       DocEntry *entry = new DocEntry( desktopFile.readName(), url.url(), icon );
-      NavigatorItem *item = new NavigatorItem( entry, topItem );
+      NavigatorItem *item = new NavigatorAppItem( entry, topItem );
       item->setAutoDeleteDocEntry( true );
     }
 }
@@ -309,16 +285,15 @@ void Navigator::insertInfoDocs( NavigatorItem *topItem )
   infoTree->build( topItem );
 }
 
-NavigatorItem *Navigator::insertScrollKeeperDocs( NavigatorItem *topItem,
-                                                  NavigatorItem *after )
+void Navigator::insertScrollKeeperDocs( NavigatorItem *topItem )
 {
   ScrollKeeperTreeBuilder *builder = new ScrollKeeperTreeBuilder( this );
-  return builder->build( topItem, after );
+  builder->build( topItem );
 }
 
 void Navigator::selectItem( const QUrl &url )
 {
-  qDebug() << "Navigator::selectItem(): " << url.url();
+  khcDebug() << "Navigator::selectItem(): " << url.url();
 
   if ( url.url() == "khelpcenter:home" ) {
     clearSelection();
@@ -341,17 +316,17 @@ void Navigator::selectItem( const QUrl &url )
   if ( item && mSelected ) {
     QUrl currentURL ( item->entry()->url() );
     if ( (currentURL == url) || (currentURL == alternativeURL) ) {
-      qCDebug(category) << "URL already shown.";
+      khcDebug() << "URL already shown.";
       return;
     }
   }
 
-  // First, populate the NavigatorAppItems if we don't want the home page
+  // First, populate the NavigatorAppGroupItems if we don't want the home page
   if ( url != homeURL() ) {
     QTreeWidgetItemIterator it1( mContentsTree );
     while( (*it1) ) 
     {
-      NavigatorAppItem *appItem = dynamic_cast<NavigatorAppItem *>( (*it1) );
+      NavigatorAppGroupItem *appItem = dynamic_cast<NavigatorAppGroupItem *>( (*it1) );
       if ( appItem ) appItem->populate( true );
       ++it1;
     }
@@ -392,7 +367,7 @@ void Navigator::slotItemSelected( QTreeWidgetItem *currentItem )
 
   NavigatorItem *item = static_cast<NavigatorItem *>( currentItem );
 
-  qCDebug(category) << item->entry()->name() << endl;
+  khcDebug() << item->entry()->name();
 
   item->setExpanded( !item->isExpanded() );
 
@@ -409,8 +384,18 @@ void Navigator::slotItemSelected( QTreeWidgetItem *currentItem )
    
     emit itemSelected( url.url() );
   }
+}
 
-  mLastUrl = url;
+void Navigator::slotItemExpanded( QTreeWidgetItem *item )
+{
+  NavigatorItem *ni = static_cast<NavigatorItem *>( item );
+  ni->itemExpanded( true );
+}
+
+void Navigator::slotItemCollapsed( QTreeWidgetItem *item )
+{
+  NavigatorItem *ni = static_cast<NavigatorItem *>( item );
+  ni->itemExpanded( false );
 }
 
 void Navigator::openInternalUrl( const QUrl &url )
@@ -434,18 +419,6 @@ void Navigator::showOverview( NavigatorItem *item, const QUrl &url )
 {
   mView->beginInternal( url );
 
-  QString fileName = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "khelpcenter/index.html.in" );
-  if ( fileName.isEmpty() )
-    return;
-
-  QFile file( fileName );
-
-  if ( !file.open( QIODevice::ReadOnly ) )
-    return;
-
-  QTextStream stream( &file );
-  QString res = stream.readAll();
-
   QString title,name,content;
   uint childCount;
 
@@ -469,24 +442,18 @@ void Navigator::showOverview( NavigatorItem *item, const QUrl &url )
     if ( item ) child = item;
     else child = mContentsTree->invisibleRootItem();
 
-    mDirLevel = 0;
-
-    content += createChildrenList( child );
+    content += createChildrenList( child, 0 );
   }
   else
     content += QLatin1String("<p></p>");
 
-  res = res.arg(title).arg(name).arg(content);
-
-  mView->write( res );
+  mView->write( mView->grantleeFormatter()->formatOverview( title, name, content ) );
 
   mView->end();
 }
 
-QString Navigator::createChildrenList( QTreeWidgetItem *child )
+QString Navigator::createChildrenList( QTreeWidgetItem *child, int level )
 {
-  ++mDirLevel;
-
   QString t;
 
   t += QLatin1String("<ul>\n");
@@ -508,8 +475,8 @@ QString Navigator::createChildrenList( QTreeWidgetItem *child )
       t += QLatin1String("<br>") + e->info();
     }
 
-    if ( childItem->childCount() > 0 && mDirLevel < 2 ) {
-      t += createChildrenList( childItem );
+    if ( childItem->childCount() > 0 && level < 1 ) {
+      t += createChildrenList( childItem, level + 1 );
     }
 
     t += QLatin1String("</li>\n");
@@ -518,27 +485,25 @@ QString Navigator::createChildrenList( QTreeWidgetItem *child )
 
   t += QLatin1String("</ul>\n");
 
-  --mDirLevel;
-
   return t;
 }
 
 void Navigator::slotSearch()
 {
   
-  qCDebug(category) << "Navigator::slotSearch()";
+  khcDebug() << "Navigator::slotSearch()";
 
-  if ( !checkSearchIndex() ) return;
+  if ( mIndexingProc ) return;
 
   if ( mSearchEngine->isRunning() ) return;
 
   QString words = mSearchEdit->text();
   QString method = mSearchWidget->method();
   int pages = mSearchWidget->pages();
-  QString scope = mSearchWidget->scope();
+  QStringList scope = mSearchWidget->scope();
 
-  qCDebug(category) << "Navigator::slotSearch() words: " << words;
-  qCDebug(category) << "Navigator::slotSearch() scope: " << scope;
+  khcDebug() << "Navigator::slotSearch() words: " << words;
+  khcDebug() << "Navigator::slotSearch() scope: " << scope;
 
   if ( words.isEmpty() || scope.isEmpty() ) return;
 
@@ -566,50 +531,23 @@ void Navigator::slotSearchFinished()
   mSearchButton->setEnabled(true);
   QApplication::restoreOverrideCursor();
 
-  qCDebug(category) << "Search finished.";
+  khcDebug() << "Search finished.";
 }
 
 void Navigator::checkSearchButton()
 {
   mSearchButton->setEnabled( !mSearchEdit->text().isEmpty() &&
-    mSearchWidget->scopeCount() > 0 );
+    mSearchWidget->scopeCount() > 0 && !mIndexingProc );
   mTabWidget->setCurrentIndex( mTabWidget->indexOf( mSearchWidget ) );
 }
 
 
 void Navigator::hideSearch()
 {
-  mSearchFrame->hide();
+  mSearchLineEditWidget->hide();
   mTabWidget->removeTab( mTabWidget->indexOf( mSearchWidget ) );
 }
 
-bool Navigator::checkSearchIndex()
-{
-  KConfigGroup cfg(KSharedConfig::openConfig(), "Search" );
-  if ( cfg.readEntry( "IndexExists", false) ) return true;
-
-  if ( mIndexDialog && !mIndexDialog->isHidden() ) return true;
-
-  QString text = i18n( "A search index does not yet exist. Do you want "
-                       "to create the index now?" );
-
-  int result = KMessageBox::questionYesNo( this, text, QString(),
-                                           KGuiItem(i18n("Create")),
-                                           KGuiItem(i18n("Do Not Create")),
-                                           QLatin1String("indexcreation") );
-  if ( result == KMessageBox::Yes ) {
-    showIndexDialog();
-    return false;
-  }
-
-  return true;
-}
-
-void Navigator::slotTabChanged( int id )
-{
-  if ( mTabWidget->widget(id) == mSearchWidget ) checkSearchIndex();
-}
-
 void Navigator::slotSelectGlossEntry( const QString &id )
 {
   mGlossaryTree->slotSelectGlossEntry( id );
@@ -627,16 +565,6 @@ QUrl Navigator::homeURL()
   return mHomeUrl;
 }
 
-void Navigator::showIndexDialog()
-{
-  if ( !mIndexDialog ) {
-    mIndexDialog = new KCMHelpCenter( mSearchEngine, this );
-    connect(mIndexDialog, &KCMHelpCenter::searchIndexUpdated, mSearchWidget, &SearchWidget::updateScopeList);
-  }
-  mIndexDialog->show();
-  mIndexDialog->raise();
-}
-
 void Navigator::readConfig()
 {
   if ( Prefs::currentTab() == Prefs::Search ) {
@@ -664,6 +592,110 @@ void Navigator::clearSearch()
   mSearchEdit->setText( QString() );
 }
 
+void Navigator::slotDelayedIndexingStart()
+{
+  mIndexingQueue.clear();
+
+  const DocEntry::List &entries = DocMetaInfo::self()->docEntries();
+  foreach ( DocEntry *entry, entries ) {
+    if ( mSearchEngine->needsIndex( entry ) ) {
+      mIndexingQueue.append( entry );
+    }
+  }
+
+  if ( mIndexingQueue.isEmpty() ) {
+    return;
+  }
+
+  emit setStatusBarText( i18n( "Updating search index..." ) );
+
+  mIndexingTimer.start();
+
+  slotDoIndexWork();
+}
+
+void Navigator::slotDoIndexWork()
+{
+  if ( mIndexingQueue.isEmpty() ) {
+    mIndexingTimer.stop();
+    emit setStatusBarText( i18n( "Updating search index... done." ) );
+    mIndexingBar->hide();
+    mSearchWidget->searchIndexUpdated();
+    return;
+  }
+
+  const DocEntry *entry = mIndexingQueue.takeFirst();
+
+  QString error;
+  SearchHandler *handler = mSearchEngine->handler( entry->documentType() );
+  if ( !handler ) {
+    return slotDoIndexWork();
+  }
+  if ( !handler->checkPaths( &error ) ) {
+    khcWarning() << "Indexing path error for" << entry->name() << ":" << error;
+    return slotDoIndexWork();
+  }
+  QString indexer = handler->indexCommand( entry->identifier() );
+  if ( indexer.isEmpty() ) {
+    khcWarning() << "Empty indexer for" << entry->identifier() << entry->documentType();
+    return slotDoIndexWork();
+  }
+
+  const QString indexDir = Prefs::indexDirectory();
+
+  indexer.replace( QLatin1String( "%i" ), entry->identifier() );
+  indexer.replace( QLatin1String( "%d" ), indexDir );
+  indexer.replace( QLatin1String( "%p" ), entry->url() );
+  khcDebug() << "Indexer:" << indexer;
+
+  if ( !QDir().mkpath( indexDir ) ) {
+    khcWarning() << "cannot create the directory:" << indexDir;
+    return slotDoIndexWork();
+  }
+
+  mIndexingProc = new KProcess;
+
+  *mIndexingProc << KShell::splitArgs( indexer );
+
+  connect( mIndexingProc, SIGNAL( finished( int, QProcess::ExitStatus) ),
+           SLOT( slotProcessExited( int, QProcess::ExitStatus) ) );
+
+  mIndexingProc->start();
+
+  if ( !mIndexingProc->waitForStarted() )  {
+    khcWarning() << "Unable to start command" << indexer;
+    delete mIndexingProc;
+    mIndexingProc = 0;
+    return slotDoIndexWork();
+  }
+}
+
+void Navigator::slotProcessExited( int exitCode, QProcess::ExitStatus exitStatus )
+{
+  if ( exitStatus != QProcess::NormalExit ) {
+    khcWarning() << "Process failed";
+    khcWarning() << "stdout output:" << mIndexingProc->readAllStandardOutput();
+    khcWarning() << "stderr output:" << mIndexingProc->readAllStandardError();
+  } else if ( exitCode != 0 ) {
+    khcWarning() << "running" << mIndexingProc->program() << "failed with exitCode" << exitCode;
+    khcWarning() << "stdout output:" << mIndexingProc->readAllStandardOutput();
+    khcWarning() << "stderr output:" << mIndexingProc->readAllStandardError();
+  }
+  delete mIndexingProc;
+  mIndexingProc = 0;
+
+  slotDoIndexWork();
+}
+
+void Navigator::slotShowIndexingProgressBar()
+{
+  if ( !mIndexingProc ) {
+    return;
+  }
+
+  mIndexingBar->setRange( 0, 0 );
+  mIndexingBar->show();
+}
 
 
 // vim:ts=2:sw=2:et
